/*
 * This file is part of gms
 * Copyright (C) 2021 Istituto Nazionale di Astrofisica
 * SPDX-License-Identifier: GPL-3.0-or-later
 */
package it.inaf.ia2.gms.model;

public enum Permission {

    ADMIN,
    MANAGE_MEMBERS,
    VIEW_MEMBERS,
    TRAVERSE;

    /**
     * Updates the permission keeping the priority (e.g. MANAGE_MEMBERS
     * overrides VIEW_MEMBERS, but not the opposite).
     */
    public static Permission addPermission(Permission oldPermission, Permission newPermission) {

        if (oldPermission == null) {
            return newPermission;
        }

        switch (newPermission) {
            case ADMIN:
                return ADMIN;
            case MANAGE_MEMBERS:
                if (oldPermission != Permission.ADMIN) {
                    return MANAGE_MEMBERS;
                }
            case VIEW_MEMBERS:
                if (oldPermission == Permission.TRAVERSE) {
                    return VIEW_MEMBERS;
                }
        }

        return oldPermission;
    }

    public static boolean includes(Permission permission, Permission permissionToCheck) {
        if (permissionToCheck == null) {
            throw new IllegalArgumentException("Permission to check cannot be null");
        }

        if (permission == null) {
            return false;
        }
        switch (permissionToCheck) {
            case ADMIN:
                return permission == ADMIN;
            case MANAGE_MEMBERS:
                return permission == MANAGE_MEMBERS || permission == ADMIN;
            case VIEW_MEMBERS:
                return permission != TRAVERSE;
            case TRAVERSE:
                return true;
        }
        return false;
    }
}
